@@ -3,7 +3,7 @@
 # This file is part of Hypothesis, which may be found at
 # https://github.com/HypothesisWorks/hypothesis-python
 #
-# Most of this work is copyright (C) 2013-2017 David R. MacIver
+# Most of this work is copyright (C) 2013-2018 David R. MacIver
 # (david@drmaciver.com), but it contains contributions by others. See
 # CONTRIBUTING.rst for a full list of people who may hold copyright, and
 # consult the git log if you need to determine who owns an individual
@@ -21,7 +21,6 @@ a single value.
 
 Notably, the set of steps available at any point may depend on the
 execution to date.
-
 """
 
 
@@ -38,19 +37,22 @@ from hypothesis.core import find
 from hypothesis.errors import Flaky, NoSuchExample, InvalidDefinition, \
     HypothesisException
 from hypothesis.control import BuildContext
-from hypothesis._settings import settings as Settings
 from hypothesis._settings import Verbosity
+from hypothesis._settings import settings as Settings
 from hypothesis.reporting import report, verbose_report, current_verbosity
 from hypothesis.strategies import just, lists, builds, one_of, runner, \
     integers
 from hypothesis.vendor.pretty import CUnicodeIO, RepresentationPrinter
 from hypothesis.internal.reflection import proxies, nicerepr
 from hypothesis.internal.conjecture.data import StopTest
-from hypothesis.internal.conjecture.utils import integer_range
+from hypothesis.internal.conjecture.utils import integer_range, \
+    calc_label_from_name
 from hypothesis.searchstrategy.strategies import SearchStrategy
 from hypothesis.searchstrategy.collections import TupleStrategy, \
     FixedKeysDictStrategy
 
+STATE_MACHINE_RUN_LABEL = calc_label_from_name('another state machine step')
+
 
 class TestCaseProperty(object):  # pragma: no cover
 
@@ -99,7 +101,6 @@ def run_state_machine_as_test(state_machine_factory, settings=None):
     state_machine_factory is anything which returns an instance of
     GenericStateMachine when called with no arguments - it can be a class or a
     function. settings will be used to control the execution of the test.
-
     """
     try:
         breaker = find_breaking_runner(state_machine_factory, settings)
@@ -116,7 +117,6 @@ def run_state_machine_as_test(state_machine_factory, settings=None):
 
 
 class GenericStateMachine(object):
-
     """A GenericStateMachine is the basic entry point into Hypothesis's
     approach to stateful testing.
 
@@ -136,7 +136,6 @@ class GenericStateMachine(object):
 
     And if this ever produces an error it will shrink it down to a small
     sequence of example choices demonstrating that.
-
     """
 
     def steps(self):
@@ -152,7 +151,6 @@ class GenericStateMachine(object):
         """Print a step to the current reporter.
 
         This is called right before a step is executed.
-
         """
         self.step_count = getattr(self, u'step_count', 0) + 1
         report(u'Step #%d: %s' % (self.step_count, nicerepr(step)))
@@ -162,7 +160,6 @@ class GenericStateMachine(object):
         state.
 
         Does nothing by default
-
         """
         pass
 
@@ -211,11 +208,9 @@ GenericStateMachine.find_breaking_runner = classmethod(find_breaking_runner)
 
 
 class StateMachineRunner(object):
-
     """A StateMachineRunner is a description of how to run a state machine.
 
     It contains values that it will use to shape the examples.
-
     """
 
     def __init__(self, data, n_steps):
@@ -236,7 +231,7 @@ class StateMachineRunner(object):
             while True:
                 if steps >= self.n_steps:
                     stopping_value = 0
-                self.data.start_example()
+                self.data.start_example(STATE_MACHINE_RUN_LABEL)
                 if not cu.biased_coin(self.data, stopping_value):
                     self.data.stop_example()
                     break
@@ -303,7 +298,6 @@ def rule(targets=(), target=None, **kwargs):
     invocation. If their value is a Bundle then values that have previously
     been produced for that bundle will be provided, if they are anything else
     it will be turned into a strategy and values from that will be provided.
-
     """
     if target is not None:
         targets += (target,)
@@ -358,7 +352,6 @@ def precondition(precond):
 
     This is better than using assume in your rule since more valid rules
     should be able to be run.
-
     """
     def decorator(f):
         @proxies(f)
@@ -402,7 +395,6 @@ def invariant():
             @invariant()
             def is_nonzero(self):
                 assert self.state != 0
-
     """
     def accept(f):
         existing_invariant = getattr(f, INVARIANT_MARKER, None)
@@ -430,7 +422,6 @@ class ShuffleBundle(object):
 
 
 class RuleBasedStateMachine(GenericStateMachine):
-
     """A RuleBasedStateMachine gives you a more structured way to define state
     machines.
 
@@ -439,8 +430,8 @@ class RuleBasedStateMachine(GenericStateMachine):
     from bundles (or just from normal strategies) and push data onto
     bundles. At any given point a random applicable rule will be
     executed.
-
     """
+
     _rules_per_class = {}
     _invariants_per_class = {}
     _base_rules_per_class = {}
